#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "mesh.h"


int loadGIS(pMesh mesh,const char *namegis,ubyte quiet) {
  pPoint   ppt;
  FILE    *fp;
  float   *te,coef;
  int      i,j,ni,ret,bitsize,pos;
  char    *ptr,c,buf[256],data[256];
  ubyte    ityp;
  

  fprintf(stdout," Loading terrain: %s\n",namegis);
  ptr = strstr(namegis,".gis");
  strcpy(data,namegis);
  if ( !ptr )  strcat(data,".gis");
  fp = fopen(data,"rb");
  if ( !fp ) {
    fprintf(stderr,"  ## UNABLE TO OPEN FILE %s.\n",data);
    return(0);
  }
  if ( !quiet )
    fprintf(stdout,"  opening %s\n",data);

  if ( !fgets(buf,sizeof(buf),fp) ) {
    fprintf(stderr,"  ## INVALID HEADER.\n");
    return(0);
  }

  /* remove leading spaces */
  pos = 0;
  while ( (buf[0] == ' ') && (buf[0] != 0x0d) && (buf[0] != 0x0a) )
    memmove(&buf[0],&buf[1],strlen(buf));

  /* check header file */
  if ( buf[0] != 'G' ) {
    fprintf(stderr,"  ## INVALID TERRAIN FORMAT.\n");
    return(0);
  }
  if ( buf[1] == '1' )       ityp = 1;
  else if ( buf[1] == '2' )  ityp = 2;
  else {
    fprintf(stderr,"  ## INVALID TERRAIN FORMAT (MUST BE 'G[12]').\n");
    return(0);
  }

  /* check and strip comments */
  do {
    ret = fscanf(fp,"%s",&buf);
    if ( ret == EOF ) break;
    if ( buf[0] == '#' )
      do
        c = getc(fp);
      while ( c != '\n' );
    else break;
  }
  while (1);

  /* header */
  ret  = sscanf(buf,"%d",&mesh->sy);
  ret += fscanf(fp,"%d",&mesh->sx);
  ret += fscanf(fp,"%f %f %f",&mesh->cx,&mesh->cy,&mesh->cz);
  ret += fscanf(fp,"%lf",&mesh->gu);
  ret += fscanf(fp,"%lf",&mesh->hu);
  ret += fscanf(fp,"%f %f",&mesh->xmi,&mesh->ymi);
  if ( ret != 9 ) {
    fprintf(stderr,"  ## ERROR LOADING TERRAIN. %d\n",ret);
    free(mesh);
    return(0);
  }

  /* mesh structure */
  mesh->npfixe = mesh->sx*mesh->sy;
  bitsize      = mesh->npfixe*sizeof(float);
  mesh->point  = (pPoint)calloc(mesh->npfixe+1,sizeof(Point));
  if ( !mesh->point ) {
    fprintf(stderr,"  Sorry, not enough memory. Bye.\n");
    free(mesh);
    return(0);
  }
  fprintf(stdout,"   terrain size: %dx%d  %ld bytes\n",mesh->sx,mesh->sy,bitsize);
  
  /* read data */
  mesh->xmin = mesh->ymin = mesh->zmin =  1.e30;
  mesh->xmax = mesh->ymax = mesh->zmax = -1.e30;
    
  if ( ityp == 1 ) {
    mesh->np = 0;
    for (i=0; i<mesh->sy; i++)
      for ( j=0; j<mesh->sx; j++) {
        ppt = &mesh->point[++mesh->np];
        ret = fscanf(fp,"%f",&ppt->z);
        if ( feof(fp) ) {
          fprintf(stderr,"  unexpected EOF. Bye\n");
          return(0);
        }
        /*
        ppt->x = (mesh->gu/mesh->cx)*(xmi/mesh->cx + j);
        ppt->y = (mesh->gu/mesh->cy)*(ymi/mesh->cy + i);
        ppt->z = (mesh->hu/mesh->cz)*ppt->z;
        */
        ppt->x = (float)j;
        ppt->y = (float)i;
        ppt->z = (mesh->hu/mesh->cz)*ppt->z * (mesh->cx/mesh->gu);
        mesh->xmin = min(mesh->xmin,ppt->x);
        mesh->xmax = max(mesh->xmax,ppt->x);
        mesh->ymin = min(mesh->ymin,ppt->y);
        mesh->ymax = max(mesh->ymax,ppt->y);
        mesh->zmin = min(mesh->zmin,ppt->z);
        mesh->zmax = max(mesh->zmax,ppt->z);
      }
  }
  else {
    te = (float*)malloc(mesh->sx*sizeof(float));
    if ( !te )  exit(1);
    mesh->np = ni = 0;
    for (i=0; i<mesh->sy; i++) {
      ret = fread(te,sizeof(int),mesh->sx,fp);
      if ( ret != mesh->sx ) {
        fprintf(stderr,"  ## ERROR LOADING TERRAIN.\n");
        free(mesh->point);
        free(mesh);
        return(0);
      }
      for (j=0; j<mesh->sx; j++) {
        ppt = &mesh->point[++mesh->np];
        ppt->z = te[ni++];
        /*
        ppt->x = ((float)mesh->gu/mesh->cx)*(xmi/mesh->cx + j);
        ppt->y = ((float)mesh->gu/mesh->cy)*(ymi/mesh->cy + i);
        ppt->z = ((float)mesh->hu/mesh->cz)*ppt->z;
        */
        ppt->x = (float)j;
        ppt->y = (float)i;
        ppt->z = (mesh->hu/mesh->cz)*ppt->z * (mesh->cx/mesh->gu);
        mesh->xmin = min(mesh->xmin,ppt->x);
        mesh->xmax = max(mesh->xmax,ppt->x);
        mesh->ymin = min(mesh->ymin,ppt->y);
        mesh->ymax = max(mesh->ymax,ppt->y);
        mesh->zmin = min(mesh->zmin,ppt->z);
        mesh->zmax = max(mesh->zmax,ppt->z);
      }
    }
    free(te);
  }

  if ( mesh->sx*mesh->sy != mesh->np ) {
    fprintf(stderr,"  Missing or erroneous data. %d %d\n",
            mesh->np,mesh->sx*mesh->sy);
    free(mesh->point);
    free(mesh);
    return(0);
  }

  if ( !quiet ) {
    printf("   normalized altitudes: %g , %g   deltaz %g\n",
           mesh->zmin,mesh->zmax,mesh->zmax-mesh->zmin);
    coef = (mesh->cz/mesh->hu)*(mesh->gu/mesh->cx)*mesh->hu/mesh->cz;
    printf("   absolute   altitudes: %g , %g   deltaz %g\n",
           coef*mesh->zmin,
           coef*mesh->zmax,          
           coef*(mesh->zmax-mesh->zmin));
  }
  return(1);  
}

